package cn.com.qmth.stmms.ms.admin.exporter;

import cn.com.qmth.stmms.ms.admin.assembler.MarkUserAssembler;
import cn.com.qmth.stmms.ms.admin.dto.MarkUserDTO;
import cn.com.qmth.stmms.ms.admin.dto.StudentDTO;
import cn.com.qmth.stmms.ms.commons.config.ImageConfig;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExportUtils;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Stream;

/**
 * 导出用户信息
 * Created by zhengmin on 2016/11/18.
 */
@RestController
@RequestMapping("api/export/users")
public class UserExporter {

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private MarkUserAssembler markUserAssembler;

    @Resource
    MarkSubjectRepo markSubjectRepo;

    @Resource
    ImageConfig imageConfig;

    @RequestMapping(method = RequestMethod.GET)
    public void export(@RequestParam Long workId, @RequestParam(required = false) Subject subject, HttpServletResponse response) {
        List<MarkUserDTO> markUserDTOs = new ArrayList<>();
        Stream<MarkUser> markUserStream = markUserRepo.findByWorkId(workId).stream();
        if (subject != null) {
            markUserStream = markUserStream.filter(u -> u.getSubject().name() == subject.name() && (Role.MARKER.equals(u.getRole()) || Role.MARK_LEADER.equals(u.getRole())));
        }
        markUserStream.forEach(u -> {
            markUserDTOs.add(markUserAssembler.toDTO(u));
        });
        String fileName = "用户列表";
        ExportUtils.exportEXCEL(fileName, MarkUserDTO.class, markUserDTOs, response);
    }

    @RequestMapping(value = "{work}/{subject}/export", method = RequestMethod.GET)
    public void countGropuByQuestion(@PathVariable Work work, @PathVariable Subject subject, HttpServletResponse response) {
        if (work != null) {
            Set<StudentDTO> list = new HashSet<>();
            // 试卷标记缺考
            List<Object[]> result = studentRepo.listMissingBySubject(work.getId(), subject.toString());
            result.forEach(i -> {
                StudentDTO studentDTO = new StudentDTO();
                studentDTO.setExamNumber(String.valueOf(i[0]));
                studentDTO.setName(String.valueOf(i[1]));
                studentDTO.setAreaName(String.valueOf(i[2]));
                studentDTO.setAreaCode(String.valueOf(i[3]));
                studentDTO.setExamRoom(String.valueOf(i[4]));
                studentDTO.setSourceName(String.valueOf(i[5]));
                studentDTO.setSchool(String.valueOf(i[6]));
                list.add(studentDTO);
            });

            // 导入缺考免单、考生管理标记缺考
            List<Student> studentList = null;
            if(Subject.SC.equals(subject)){
                studentList = studentRepo.findByWorkIdAndTestAndScMissingIsTrue(work.getId(), String.valueOf(TrialEnum.DEFAULT.getId()));
            } else if(Subject.SX.equals(subject)){
                studentList = studentRepo.findByWorkIdAndTestAndSxMissingIsTrue(work.getId(), String.valueOf(TrialEnum.DEFAULT.getId()));
            } else if(Subject.SM.equals(subject)){
                studentList = studentRepo.findByWorkIdAndTestAndSmMissingIsTrue(work.getId(), String.valueOf(TrialEnum.DEFAULT.getId()));
            }
            if(!CollectionUtils.isEmpty(studentList)){
                studentList.forEach(s -> {
                    // 去掉重复数据
                    long count = list.stream().filter(m-> Objects.equals(m.getExamNumber(), s.getExamNumber())).count();
                    if(count == 0) {
                        StudentDTO studentDTO = new StudentDTO();
                        studentDTO.setExamNumber(s.getExamNumber());
                        studentDTO.setName(s.getName());
                        studentDTO.setAreaName(s.getAreaName());
                        studentDTO.setAreaCode(s.getAreaCode());
                        studentDTO.setExamRoom(s.getExamRoom());
                        studentDTO.setSourceName(s.getSourceName());
                        studentDTO.setSchool(s.getSchool());
                        list.add(studentDTO);
                    }
                });
            }
            String fileName;
            if (!imageConfig.isCustomSubject()) {
                fileName = subject.toString() + "缺考名单";
            } else {
                MarkSubject markSubject = markSubjectRepo.findOne(work.getId() + "-" + Subject.CUSTOM.name());
                fileName = markSubject.getName() + "缺考名单";
            }
            ExportUtils.exportEXCEL(fileName, StudentDTO.class, list, response);
        }
    }
}
