package cn.com.qmth.stmms.ms.core.utils;

import cn.com.qmth.stmms.ms.commons.config.DbConfig;
import cn.com.qmth.stmms.ms.core.domain.Organization;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.repository.OrganizationRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import org.apache.http.client.utils.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import sun.security.acl.WorldGroupImpl;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @Date: 2021/5/31.
 */
@Component
public class DbBackupUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(DbBackupUtils.class);

    ExecutorService service = Executors.newFixedThreadPool(1);

    @Autowired
    DbConfig dbConfig;

    @Autowired
    OrganizationRepo organizationRepo;

    @Autowired
    WorkRepo workRepo;

    public void startBackup(Long workId, String stage) {
        Work work = workRepo.getOne(workId);
        if (Objects.nonNull(work)) {
            Organization organization = organizationRepo.getOne(work.getOrganizationId());
            if (Objects.nonNull(organization)) {
                String code = organization.getAbbreviation();
                service.submit(dbBackUp(workId, code, stage));
            }
        }
    }

    public Runnable dbBackUp(Long workId, String code, String stage) {
        return () -> {
            String path = "beifen";
            String pathSql = code + "-" + workId + "-" + stage + "-" + DateUtils.formatDate(new Date(), "yyyyMMddHHmmss") + ".sql";
            File fileSql = new File(path, pathSql);
            try {
                if (!fileSql.getParentFile().exists()) {
                    fileSql.getParentFile().mkdirs();
                }
                //创建备份sql文件
                if (!fileSql.exists()) {
                    fileSql.createNewFile();
                }
                //mysqldump -hlocalhost -uroot -p123456 db > /home/back.sql
                StringBuffer sb = new StringBuffer();
                sb.append("mysqldump");
                sb.append(" -h " + dbConfig.getHost());
                sb.append(" -u" + dbConfig.getUserName());
                sb.append(" -p" + dbConfig.getPassword());
                sb.append(" " + dbConfig.getDbName() + " >");
                sb.append(fileSql.getAbsolutePath());
                LOGGER.info("cmd命令为：{}，开始备份数据库：{}" , sb.toString(), dbConfig.getDbName());
                Runtime runtime = Runtime.getRuntime();
                Process process = runtime.exec("cmd /c" + sb.toString());
                LOGGER.info("备份成功!");
            } catch (IOException e) {
                LOGGER.info("备份失败！原因：{}", e.getMessage());
            }
        };
    }

    public static void main(String[] args) {
        System.out.println(System.getProperty("os.name"));
    }
}
