package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.utils.DbBackupUtils;
import cn.com.qmth.stmms.ms.commons.utils.RandomUtil;
import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.cache.CacheService;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 评卷任务分配
 * Created by zhengmin on 2016/9/23.
 */
@Service
public class AssignTaskService {

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private MarkerGroupRepo markerGroupRepo;

    @Autowired
    private TaskPublishSettingRepo taskPublishSettingRepo;

    @Value("${sys.config.random-bundle-size}")
    private int randomBundleSize;

    @Autowired
    RandomUtil randomUtil;
    
    @Autowired
    SqlUtil sqlUtil;

    @Autowired
    DbBackupUtils dbBackupUtils;

    @Autowired
    CacheService cacheService;

    @Autowired
    MarkUserRepo markUserRepo;

    /**
     * 单评任务模式，每个科目评卷员都需要对该科目每份试卷进行评卷
     * 比如，色彩有1000份试卷，那么每个评卷员都需要评1000份
     *
     * @param papers
     * @param markers
     */
    @Transactional
    public void assignForAll(List<Paper> papers, Iterable<MarkUser> markers, MarkSubject markSubject) throws Exception {
        List<MarkTask> markTasks = new ArrayList<>();
        int paperCount = papers.size();
        int randomBundleCount = paperCount / randomBundleSize;
        if (paperCount % randomBundleSize != 0) {
            randomBundleCount += 1;
        }
        List<MarkTask> markTaskList = markTaskRepo.findByWorkId(markSubject.getWorkId());
        Map<Long, Object> randomMap = markTaskList.stream().collect(Collectors.toMap(MarkTask::getRandomSeqNew, Function.identity(), (dto1, dto2) -> dto1));
        for (int i = 0; i < randomBundleCount; i++) {
            int fromIndex = i * randomBundleSize;
            int endIndex = i * randomBundleSize + randomBundleSize;
            if (endIndex > paperCount) {
                endIndex = paperCount;
            }
            List<Paper> bundleList = papers.subList(fromIndex, endIndex);

            for (MarkUser marker : markers) {
                List<Paper> markerPaperList = Arrays.asList(new Paper[bundleList.size()]);

                Collections.copy(markerPaperList, bundleList);
                Collections.shuffle(markerPaperList);//乱序

                for (int j = 0; j < markerPaperList.size(); j++) {
                    Paper paper = markerPaperList.get(j);
                    Long random = getRandom(marker.getId(), paper.getId(), marker.getWorkId(), paper.getExamNumber(), randomMap);
                    MarkTask markTask = new MarkTask(marker, paper, markSubject.getStage(), random, markSubject.getTest());
                    markTask.setRandomSeq(i * randomBundleSize + j + 1);
                    markTasks.add(markTask);
                }
            }

        }

        markTaskRepo.save(markTasks);

//        Iterator<Paper> iterator = papers.iterator();
//        while (iterator.hasNext()) {
//            Paper paper = iterator.next();
//            for (MarkUser marker : markers) {
//                MarkTask markTask = new MarkTask(marker, paper, markSubject.getStage());
//                markTaskRepo.save(markTask);
//                //taskExecutor.execute(papers,marker,stage);
//            }
//        }
    }

    /**
     * 分组任务模式，将该科目的评卷员分成N组，将该科目试卷平均分给这些组，每组内的评卷员每人将这些试卷打分
     * 比如：6个评卷员分2组，有1000份试卷，每组评500份，每个评卷员评500份
     *
     * @param papers
     * @param markerGroups
     * @param taskList
     */
    @Transactional
    public void assignForGrouping(List<Paper> papers, List<MarkerGroup> markerGroups, MarkSubject markSubject, List<TaskPublishSetting> taskList) throws Exception {

        List<MarkTask> markTaskList;
        List<Paper> paperList = new ArrayList<>(papers.size());

        Iterator<Paper> iterator = papers.iterator();
        //定序序号记录
        Map map = new HashMap();
        Map seqNewMap = new HashMap();
        long currentTime = System.currentTimeMillis();

        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));

        List<Object[]> markTasks = markTaskRepo.findAllByWorkId(markSubject.getWorkId());
        Map<Long, Object> randomMap = new HashMap<>();
        markTasks.forEach(m -> randomMap.put(Long.valueOf(m[1].toString()), m[0]));
        // 谁分档谁打分
        if(Objects.equals(MarkStage.SCORE, markSubject.getStage()) && ParamCache.paramMap.get(markSubject.getWorkId()).getIsLevelToScore() == 1){
            // 查询所有分档
            List<MarkTask> markTasksLevels = markTaskRepo.findByWorkIdAndSubjectAndStage(markSubject.getWorkId(),markSubject.getSubject(), MarkStage.LEVEL);
            markTaskList = new ArrayList<>(markTasksLevels.size());
            Map<Long, List<MarkTask>> markerMap = markTasksLevels.stream().collect(Collectors.groupingBy(m->m.getPaper().getId()));
            Map<String, AtomicInteger> integerMap = new HashMap<>();

            while (iterator.hasNext()) {
                Paper paper = iterator.next();
                String serialNumber = null;
                Integer displayNumber = null;
                Integer seqNewNumber = null;
                //是否定序
                long sortRule = taskList.stream().filter(m -> Objects.equals(m.getCode(), paper.getLevel()) && m.getSortRule() == 1).count();
                if (sortRule != 0) {
                    AtomicInteger atomicInteger = integerMap.get(paper.getLevel());
                    if(atomicInteger == null){
                        atomicInteger = new AtomicInteger(0);
                        integerMap.put(paper.getLevel(), atomicInteger);
                    }
                    serialNumber = paper.getLevel() + atomicInteger.incrementAndGet();
                    //是否显示序号
                    long display = taskList.stream().filter(m -> Objects.equals(m.getCode(), paper.getLevel()) && m.getDisplayNumber() == 1).count();
                    displayNumber = (int) display;

                    int seqNew = 0;
                    if (seqNewMap.containsKey(paper.getLevel())) {
                        seqNew = (int) seqNewMap.get(paper.getLevel());
                    }
                    seqNewNumber = ++seqNew;
                    seqNewMap.put(paper.getLevel(), seqNew);
                }

                List<MarkTask> markTask1 = markerMap.get(paper.getId());
                for (MarkTask mt : markTask1) {
                    MarkUser marker = new MarkUser();
                    marker.setId(mt.getMarkerId());
                    marker.setName(mt.getMarkerName());
                    marker.setWorkId(mt.getWorkId());
                    marker.setSubject(mt.getSubject());
                    Long random = getRandom(marker.getId(), paper.getId(), marker.getWorkId(), paper.getExamNumber(), randomMap);
                    MarkTask markTask = new MarkTask(marker, paper, markSubject.getStage(), random, markSubject.getTest());
                    String levelValue = String.valueOf(levelMap.get(paper.getLevel()).getLevelValue() + 1);
                    if (Objects.nonNull(serialNumber)) {
                        //和随机数位数保持一致（8位）
                        String randomSeqNew = String.format("%08d", seqNewNumber);
                        markTask.setRandomSeqNew(Long.valueOf(levelValue + randomSeqNew));
                    } else {
                        markTask.setRandomSeqNew(Long.valueOf(levelValue + markTask.getRandomSeqNew()));
                    }
                    markTask.setBatchNo(currentTime);
                    markTask.setSerialNumber(serialNumber);
                    markTask.setDisplayNumber(displayNumber);
                    markTask.setLevel(paper.getLevel());

                    markTaskList.add(markTask);
                    randomMap.put(random, markTask);
                }

                if (markSubject.getTest() != 1) {
                    if (MarkStage.SCORE.equals(markSubject.getStage())) {
                        paper.setScoreBatchNo(currentTime);
                        paperList.add(paper);
                    }
                } else {
                    paper.setBatchNo(currentTime);
                    paperList.add(paper);
                }
            }

        } else {
            markerGroups = markerGroups.stream().filter(m -> m.getMarkers().size() > 0).collect(Collectors.toList());
            int sum = markerGroups.stream().mapToInt(m -> m.getMarkers().size()).sum();
            int groupSize = markerGroups.size();
            int idx = 0;
            markTaskList = new ArrayList<>(papers.size() * sum);
            while (iterator.hasNext()) {
                if (idx >= groupSize) {
                    idx = 0;
                }
                Paper paper = iterator.next();
                String serialNumber = null;
                Integer displayNumber = null;
                Integer seqNewNumber = null;
                if (Objects.equals(MarkStage.SCORE, markSubject.getStage())) {
                    //是否定序
                    int seq = 0;
                    long sortRule = taskList.stream().filter(m -> Objects.equals(m.getCode(), paper.getLevel()) && m.getSortRule() == 1).count();
                    if (sortRule != 0) {
                        if (map.containsKey(paper.getLevel() + idx)) {
                            seq = (int) map.get(paper.getLevel() + idx);
                        }
                        seq = ++seq;
                        serialNumber = paper.getLevel() + seq;
                        //是否显示序号
                        long display = taskList.stream().filter(m -> Objects.equals(m.getCode(), paper.getLevel()) && m.getDisplayNumber() == 1).count();
                        displayNumber = (int) display;
                        //设置新值
                        map.put(paper.getLevel() + idx, seq);

                        int seqNew = 0;
                        if (seqNewMap.containsKey(paper.getLevel())) {
                            seqNew = (int) seqNewMap.get(paper.getLevel());
                        }
                        seqNewNumber = ++seqNew;
                        seqNewMap.put(paper.getLevel(), seqNew);
                    }
                }
                int markerCount = 0;
                for (MarkUser marker : markerGroups.get(idx).getMarkers()) {
                    markerCount++;
                    Long random = getRandom(marker.getId(), paper.getId(), marker.getWorkId(), paper.getExamNumber(), randomMap);
                    MarkTask markTask;
                    if (Objects.equals(MarkStage.SCORE, markSubject.getStage())) {
                        markTask = new MarkTask(marker, paper, markSubject.getStage(), random, markSubject.getTest());
                        String levelValue = String.valueOf(levelMap.get(paper.getLevel()).getLevelValue() + 1);
                        if (Objects.nonNull(serialNumber)) {
                            //和随机数位数保持一致（8位）
                            String randomSeqNew = String.format("%08d", seqNewNumber);
                            markTask.setRandomSeqNew(Long.valueOf(levelValue + randomSeqNew));
                        } else {
                            markTask.setRandomSeqNew(Long.valueOf(levelValue + markTask.getRandomSeqNew()));
                        }
                        markTask.setBatchNo(currentTime);
                        markTask.setSerialNumber(serialNumber);
                        markTask.setDisplayNumber(displayNumber);
                        markTask.setLevel(paper.getLevel());
                    } else {
                        if (paper.isSample() && paper.getLevel() != null) {
                            Level level = levelRepo.findByWorkIdAndCode(markSubject.getWorkId(), paper.getLevel());
                            markTask = new MarkTask(marker, paper, level, markSubject.getStage(), random, markSubject.getTest());

                            // 初始化排序值
                            if (markerCount == markerGroups.get(idx).getMarkers().size()) {
                                paper.setSortNum(level.getWeight() * markerCount);
                            }
                        } else {
                            markTask = new MarkTask(marker, paper, markSubject.getStage(), random, markSubject.getTest());
                        }
                        markTask.setBatchNo(currentTime);
                    }
                    markTaskList.add(markTask);
                    randomMap.put(random, markTask);
                }
                if (markSubject.getTest() != 1) {
                    if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                        //将paper置为激活
                        paper.setActive(true);
                        paper.setBatchNo(currentTime);
                        paperList.add(paper);
                        //更新subject表formal为正试评卷
//                    markSubject.setFormal(true);
//                    markSubjectRepo.saveAndFlush(markSubject);
                    }
                    if (MarkStage.SCORE.equals(markSubject.getStage())) {
                        paper.setScoreBatchNo(currentTime);
                        paperList.add(paper);
                    }
                } else {
                    paper.setBatchNo(currentTime);
                    paperList.add(paper);
                }
                idx++;
            }
        }

        List<MarkTask> data = new ArrayList<>();
        //5000条提交一次
        for (MarkTask markTask : markTaskList) {
            if (data.size() == 2000) {
                markTaskRepo.save(data);
                data.clear();
            }
            data.add(markTask);
        }
        //将剩下的数据也导入
        if (!data.isEmpty()) {
            markTaskRepo.save(data);
        }

        paperRepo.save(paperList);
        if (markSubject.getTest() != 1) {
            //更新subject表formal为正试评卷
            markSubject.setFormal(true);
            markSubjectRepo.saveAndFlush(markSubject);
        }
        //只有全部任务投放完成，才能删除分组
        /*int count = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndActiveFalseAndBatchNoIsNullAndTest(markSubject.getWorkId(), markSubject.getSubject(), 0);
        //进入分档阶段后，清空当前分组，以便打分阶段前重新分组
        if (MarkStage.LEVEL.equals(markSubject.getStage()) && count == 0) {
            for (MarkerGroup markerGroup : markerGroups) {
                markerGroup.setMarkers(null);
            }
            markerGroupRepo.delete(markerGroups);
        }*/
//        if (MarkStage.SCORE.equals(markSubject.getStage()) && count == 0) {
//            //初始化打分任务数据
//            initTaskPublishData(markSubject);
//
//        }
    }

    @Transactional
    public void assignForGroupingScore(MarkSubject markSubject, List<MarkerGroup> markerGroups) {
        if (markSubject.getTest() != 1) {
            //更新subject表formal为正试评卷
            markSubject.setFormal(true);
            markSubjectRepo.saveAndFlush(markSubject);
        }
        int count = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndActiveFalseAndBatchNoIsNullAndTest(markSubject.getWorkId(), markSubject.getSubject(), 0);
        //进入打分阶段后，清空当前分组，以便打分阶段前重新分组
        if (MarkStage.SCORE.equals(markSubject.getStage()) && count == 0) {
            for (MarkerGroup markerGroup : markerGroups) {
                markerGroup.setMarkers(null);
            }
            markerGroupRepo.delete(markerGroups);

            //初始化打分任务数据
            initTaskPublishData(markSubject);
        }

        // 所有用户强制退出
        List<MarkUser> userList = markUserRepo.findByWorkIdAndSubject(markSubject.getWorkId(), markSubject.getSubject());
        userList = userList.stream().filter(m-> Role.MARKER.equals(m.getRole()) || Role.MARK_LEADER.equals(m.getRole())).collect(Collectors.toList());
        if(!CollectionUtils.isEmpty(userList)){
            for (MarkUser user : userList) {
                cacheService.deleteTokenCache(user.getId().toString());
            }
        }
        // 备份数据库
        dbBackupUtils.startBackup(markSubject.getWorkId(), "分档");
    }

    private void initTaskPublishData(MarkSubject markSubject) {
        StringBuffer sql = new StringBuffer();
        sql.append("  SELECT                                                                    ");
        sql.append("          p.work_id workId,                                                  ");
        sql.append("          p.question_id questionId,                                          ");
        sql.append("          p.subject,                                                         ");
        sql.append("          p.level,                                                           ");
        sql.append("              COUNT(1) AS totalCount                                         ");
//        sql.append("              SUM(IF(p.score IS NOT NULL, 1, 0)) successCount,               ");
//        sql.append("              SUM(IF(p.score IS NULL, 1, 0)) waitCount                        ");
        sql.append("      FROM                                                                   ");
        sql.append("          paper p                                                            ");
        sql.append("      WHERE                                                                  ");
        sql.append("          p.work_id = '"+markSubject.getWorkId()+"' AND p.subject = '"+markSubject.getSubject().name()+"'                                    ");
        sql.append("              AND p.is_test = 0                                              ");
        sql.append("              AND p.level IS NOT NULL                                        ");
        sql.append("              AND p.is_missing = FALSE                                       ");
        sql.append("              AND p.is_active = TRUE                                         ");
        sql.append("      GROUP BY p.work_id , p.question_id, p.subject , p.level                ");
        sql.append("  ORDER BY p.level                                                            ");
        List list =  sqlUtil.execSqlForMap(sql.toString());
        List<TaskPublishSetting> taskPublishSettings = new ArrayList<>(list.size());
        for (Object o : list) {
            Map map = (Map) o;
            TaskPublishSetting taskPublishSetting = new TaskPublishSetting();
            taskPublishSetting.setWorkId(Long.valueOf(map.get("workId").toString()));
            taskPublishSetting.setQuestionId(Long.valueOf(map.get("questionId").toString()));
            taskPublishSetting.setSubject((String) map.get("subject"));
            taskPublishSetting.setCode((String) map.get("level"));
            taskPublishSetting.setTotalCount(Integer.valueOf(map.get("totalCount").toString()));
            taskPublishSetting.setSuccessCount(0);
            taskPublishSetting.setWaitCount(Integer.valueOf(map.get("totalCount").toString()));
            taskPublishSetting.setSortRule(0);
            taskPublishSetting.setPublishCount(0);
            taskPublishSettings.add(taskPublishSetting);
        }
        taskPublishSettingRepo.save(taskPublishSettings);
    }

    /**
     * 获取随机号
     *
     * @param markerId
     * @param paperId
     * @param workId
     * @param examNumber
     * @param randomMap
     * @return
     * @throws Exception
     */
    public Long getRandom(Long markerId, Long paperId, Long workId, String examNumber, Map<Long, Object> randomMap) throws Exception {
        int count = 0;
        Long random = 0L;
        while (true) {
            random = randomUtil.getRandomMap().get(workId).get(new Random().nextInt(randomUtil.getRandomMap().get(workId).size()));
            if (Objects.isNull(randomMap) || randomMap.size() == 0) {
                break;
            }
            if (Objects.isNull(randomMap.get(random))) {
                break;
            } else {
                count++;
            }
            if (count > 1000) {
//                throw new Exception("重复几率较高，建议重新生成随机号");
                randomUtil.getRandom(workId, true);
                getRandom(markerId, paperId, workId, examNumber, randomMap);
            }
        }
        return random;
    }
}
